#!/usr/bin/env python
import serial, os
import time
import sys, tty, termios
import math

#conf:
port = "/dev/ttyS0"
ser = serial.Serial(port, 115200, timeout=0)
res=[0,0,0,0]; 

def _GetchUnix():
	fd = sys.stdin.fileno()
	old_settings = termios.tcgetattr(fd)
	try:
		tty.setraw(sys.stdin.fileno())
		ch = sys.stdin.read(1)
	finally:
		termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
	return ch

def clear_n_lines(n):
	clr='\033[%dA'%n
	for i in range(n):
		clr+='\033[2K\n'
	clr+='\033[%dA'%(n+1)
	print clr
	
def program_fpga():
	#os.system('./mem_update.bash -b mc8051_top.bit -p')
	os.system('./mem_update.bash -b adc209_daq.bit -m adc209_daq.mem -p')

	print 
	print
	print
	print 'ADC209_DAQ v1.0'	
	print

def set_static_step(step):
	print 'Actual static step=%.3f'%step
	stepnew=raw_input('New step=')
	try:
		instepfloat = float(stepnew)
		step=instepfloat
		print 'Set new step=%.3f'%step
	except ValueError:
		print '%s is not a number!'%stepnew
	print 'Press any key...'
	getch=_GetchUnix()
	clear_n_lines(5)
	print
	return step

def get_static_single_step():
	ser.write('p')
	getloop=True
	fifo = ""
	while getloop:
		x=ser.readline()
		fifo+=x
		while len(fifo)>=4:
			tmp = ord(fifo[0]) >> 1
			point = 64*(tmp & 63)
			tmp = ord(fifo[1]) >> 1
			point = point + (tmp & 63)
			tmp = ord(fifo[2]) >> 1
			value = 64*(tmp & 3)
			tmp = ord(fifo[3]) >> 1
			value = value + (tmp & 63)
			#print "%d %d %d %d -------- %d %d"%(ord(fifo[0]),ord(fifo[1]),ord(fifo[2]),ord(fifo[3]),pointa,valuea)
			if ord(fifo[0])==129 and ord(fifo[1])==129 and ord(fifo[2])==129 and ord(fifo[3])==129:
				print 'Measurements aborted!'
				print 'Press any key...'
				getch=_GetchUnix()
				clear_n_lines(2)
				return 0
				getloop=False
			else:
				return value
				getloop=False
			fifo = fifo[4:]

def load_data_from_file(filename):
	val_list=[]
	fin=open(filename,'r')
	lines=fin.readlines()
	for line in lines:
		volt=float(line.split('\t')[0].strip())
		code=line.split('\t')[1].strip()
		if code=='?':
			codeint=-1
		else:
			codeint=int(code)
		val_list.append([volt,codeint])
	step=val_list[1][0]-val_list[0][0]
	return (step,len(val_list)-1,val_list)
	
			
def save_data_to_file(filename,step,values):
	fout=open(filename,'w')
	#fout.write('step=%d\n'%step)
	for i in values:
		if i[1]<0:
			code='?'
		else:
			code='%d'%i[1]
		fout.write('%.3f\t%s\n'%(i[0],code))
	fout.close()

def save_autodata_to_file(filename,points,values):
	fout=open(filename,'w')
	#fout.write('Point Value\n')
	index = 0
	for i in values:
		fout.write('%d\n'%values[index])
		index+=1
	fout.close()
	
def save_autodata_to_file_sorted(filename,points,values):
	fout=open('%s_sorted'%filename,'w')
	#fout.write('Point Value\n')
	index = 0
	values.sort()
	for i in values:
		fout.write('%d\t%d\n'%(index,values[index]))
		index+=1
	fout.close()
	
def get_data_auto():
	filename=''
	points=[]
	values=[] 
	expected_point=0
	fifo = ""
	getloop=True
	while getloop:
		x=ser.readline()
		fifo+=x
		while len(fifo)>=4:
			tmp = ord(fifo[0]) >> 1
			point = 64*(tmp & 63)
			tmp = ord(fifo[1]) >> 1
			point = point + (tmp & 63)
			tmp = ord(fifo[2]) >> 1
			value = 64*(tmp & 3)
			tmp = ord(fifo[3]) >> 1
			value = value + (tmp & 63)

			#print "%d %d %d %d -------- %d %d"%(ord(fifo[0]),ord(fifo[1]),ord(fifo[2]),ord(fifo[3]),point,value)
			if ord(fifo[0])==128 and ord(fifo[1])==128 and ord(fifo[2])==128 and ord(fifo[3])==128:
				print 'Measurements finished. Enter filename and press Enter:'
				filename=raw_input('Filename: ')
				#filename='fft'
				clear_n_lines(2)
				save_autodata_to_file(filename,points,values)
				getloop=False
			elif ord(fifo[0])==129 or ord(fifo[1])==129 or ord(fifo[2])==129 or ord(fifo[3])==129:
				print 'Measurements aborted!'
				print 'Press any key...'
				getch=_GetchUnix()
				clear_n_lines(2)
				getloop=False
			else:
				if point == expected_point:
					points.append(point)
					values.append(value)
					print "[%d] = %d"%(point,value)
					expected_point+=1
				else:
					print 'Data point missing - rerun measurements!'
					print 'Press any key...'
					getch=_GetchUnix()
					clear_n_lines(2)
					getloop=False
			fifo = fifo[4:]
	return filename
	
	
def get_static_data(step):
	if step==0.0:
		print 'Error: Step is equal 0.0'
		print 'Press any key...'
		getch=_GetchUnix()
		clear_n_lines(2)
		return 0
		
	val_list=[]
	volts=0.
	while volts<=3.3:
		val_list.append([volts,-1])
		volts+=step
	
	index=0
	total=len(val_list)-1
	filename="adc209_static.dat"
	getloop=True
	while getloop:
		print 'Working file: %s'%filename
		print 'Point %d of %d'%(index+1,total+1)
		print 'Enter - get data point'
		print 's   - set working file'
		print 'l   - load data from file'
		print '< , - previous point'
		print '> . - next point'
		print 'x   - to main menu'
		print
		
		for i in range(index-5,index+6):
			if i<0:
				j=total+i+1
			elif i>total:
				j=i-total-1
			else:
				j=i
			
			if val_list[j][1]<0:
				code='?'
			else:
				code='%d'%val_list[j][1]
			if j==index:
				print '\033[7m> %.3f = %s\033[0m'%(val_list[j][0],code)
			else:
				print '  %.3f = %s'%(val_list[j][0],code)
		
		print
		
		getch=_GetchUnix()
		clear_n_lines(21)
		
		if ord(getch)==13:
			if filename.strip() == "":
				print 'Set working file first!'
				print 'Press any key...'
				getch=_GetchUnix()
				clear_n_lines(2)
			else:
				val_list[index][1]=get_static_single_step()
				if index==total:
					index=0
				else:
					index+=1
				save_data_to_file(filename,step,val_list)
		
		if getch=='s':
			print 'Enter working file name (save):'
			tmp=raw_input('Filename: ')
			clear_n_lines(2)
			if tmp.strip() == "":
				print 'Empty filename'
				print 'Press any key...'
				getch=_GetchUnix()
				clear_n_lines(2)
			else:
				filename=tmp
		if getch=='l':
			print 'Enter working file name with static manual data:'
			filename=raw_input('Filename: ')
			clear_n_lines(2)
			if os.path.isfile(filename):
				(step,total,val_list)=load_data_from_file(filename)
			else:
				print 'File not exists!'
				print 'Press any key...'
				getch=_GetchUnix()
				clear_n_lines(2)
		if getch=='x':
			getloop=False
		
		if getch==',' or getch=='<':
			if index>0:
				index-=1
			else:
				index=total
		
		if getch=='.' or getch=='>':
			if index==total:
				index=0
			else:
				index+=1
	
	return step
	#clear_n_lines(3)
	
def print_raw():
	print 'CTRL+C to exit'
	print
	while True:
		print '   ADC output = %d'%get_static_single_step()
		time.sleep(0.1)
		clear_n_lines(1)
	return 0

static_step=0.005
mainloop=True
print 'ADC209_DAQ v1.0'	
print
while mainloop:
	print 'f - program FPGA'
	print 'r - get raw data'
	print 'l - set static step'
	print 'p - get static data (manual)'
	print 's - get static data (automatic)'
	print 'd - get dynamic data (automatic)'
	print 'x - exit'

	getch=_GetchUnix()
	clear_n_lines(7)
	
	if getch=='x':
		print '\033[2K\n\033[2K\n\033[2K\n\033[2K\n\033[2K\n'
		mainloop=False
	
		
	if getch=='f':
		program_fpga()
	
	if getch=='l':
		static_step=set_static_step(static_step)
		
	if getch=='p':
		static_step=get_static_data(static_step)
		
	if getch=='s':
		ser.write('s')
		get_data_auto()
		
	if getch=='d':
		ser.write('d')
		get_data_auto()
	
	if getch=='r':
		print_raw()
		
ser.close()
